// src/index.ts
import { nanoid } from "nanoid/non-secure";

// src/utils.ts
function error(error2) {
  console.error(`[MCHE Error] ${error2.message}`);
  console.error(error2.stack);
}
function log(...args) {
  console.log("[MCHE Log]", ...args);
}

// src/connection.ts
var ICE_SERVERS = [
  {
    urls: "stun:stun.l.google.com:19302"
  }
];
var MCHEConnection = class {
  #peerConnection;
  #datachannel = null;
  #datachannelCallbacks = [];
  #debug;
  constructor(onDataChannelMessage, debug, customIceServers) {
    this.#peerConnection = new RTCPeerConnection({
      iceServers: [
        ...customIceServers ?? [],
        ...ICE_SERVERS
      ]
    });
    this.#peerConnection.ondatachannel = (event) => {
      const channel = this.#datachannel = event.channel;
      if (this.#debug)
        log("RemoteDataChannel created.", channel);
      this.#datachannelCallbacks.forEach((callback) => {
        callback(channel);
      });
    };
    this.#datachannelCallbacks.push((channel) => {
      channel.onmessage = (event) => {
        onDataChannelMessage(event);
      };
    });
    this.#debug = debug;
  }
  async createOffer() {
    const channel = this.#datachannel = this.#peerConnection.createDataChannel("datachannel");
    if (this.#debug)
      log("LocalDataChannel created.", this.#datachannel);
    this.#datachannelCallbacks.forEach((callback) => {
      callback(channel);
    });
    const offer = await this.#peerConnection.createOffer();
    await this.#peerConnection.setLocalDescription(offer);
    return offer;
  }
  async receiveOffer(offer) {
    await this.#peerConnection.setRemoteDescription(offer);
  }
  async createAnswer() {
    const answer = await this.#peerConnection.createAnswer();
    await this.#peerConnection.setLocalDescription(answer);
    return answer;
  }
  async receiveAnswer(answer) {
    await this.#peerConnection.setRemoteDescription(answer);
  }
  async addIceCandidate(candidate) {
    await this.#peerConnection.addIceCandidate(candidate);
  }
  async createDataChannel(label) {
    const channel = this.#peerConnection.createDataChannel(label);
    return channel;
  }
  onCandidate(callback) {
    this.#peerConnection.onicecandidate = (event) => {
      if (event.candidate)
        callback(event.candidate);
    };
  }
  addDataChannelCallback(callback) {
    if (this.#datachannel) {
      callback(this.#datachannel);
      return;
    }
    this.#datachannelCallbacks.push(callback);
  }
  close() {
    this.#peerConnection.close();
    this.#datachannel?.close();
    this.#datachannelCallbacks.length = 0;
  }
};

// src/signaling.ts
import { createWSHE } from "wshe";
var SignalingServerClient = class {
  #ws;
  #roomId;
  constructor(url, roomId) {
    this.#ws = createWSHE(url, {
      immediate: true
    });
    this.#roomId = roomId;
  }
  send(topic, data) {
    this.#ws.send(topic, {
      ...data,
      roomId: this.#roomId
    });
  }
  on(topic, callback) {
    this.#ws.subscribe(topic, (data) => {
      if (data.roomId !== this.#roomId)
        return;
      callback(data);
    });
  }
  close() {
    this.#ws.close();
  }
};

// src/index.ts
var randomId = () => nanoid(6);
var MCHelper = class {
  #options;
  #id;
  #connectionPool = /* @__PURE__ */ new Map();
  #signallingServer;
  #dataBuffer = [];
  #dataBufferCallbacks = [];
  constructor(options) {
    this.#options = options;
    this.#id = options.id || randomId();
    this.#signallingServer = new SignalingServerClient(options.signalingServerUrl, options.roomId);
    this.#bindEvents().catch(error);
    this.#bindSignalingEvents();
  }
  #bindSignalingEvents() {
    this.#signallingServer.on("open", ({ ids }) => {
      if (this.#options.debug)
        log("Open connection with", ids);
      ids.forEach((id) => {
        this.#registerRemote(id);
      });
    });
    this.#signallingServer.on("close", ({ id }) => {
      const connection = this.#connectionPool.get(id);
      if (connection) {
        if (this.#options.debug)
          log("Close connection with", id);
        connection.close();
        this.#connectionPool.delete(id);
      }
    });
  }
  #addMessageToBuffer(message) {
    this.#dataBuffer.push(message);
    this.#dataBufferCallbacks.forEach((callback) => {
      callback(message);
      this.#dataBuffer.length = 0;
    });
  }
  #getConnection() {
    return new MCHEConnection((message) => {
      this.#addMessageToBuffer(message);
    }, this.#options.debug, this.#options.iceServers);
  }
  async #bindEvents() {
    this.#signallingServer.on("mche:offer:send" /* OfferSend */, async ({ receiver, offer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received offer from", sender, "payload is", offer);
      let connection = this.#connectionPool.get(sender);
      if (!connection) {
        connection = this.#getConnection();
        this.#connectionPool.set(sender, connection);
      }
      await connection.receiveOffer(offer);
      const answer = await connection.createAnswer();
      const payload = {
        answer,
        // swap sender and receiver
        receiver: sender,
        sender: receiver
      };
      if (this.#options.debug)
        log("Sending answer to", sender, "payload is", payload);
      this.#signallingServer.send("mche:answer:send" /* AnswerSend */, payload);
    });
    this.#signallingServer.on("mche:answer:send" /* AnswerSend */, async ({ receiver, answer, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received answer from", sender, "payload is", answer);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.receiveAnswer(answer);
    });
    this.#signallingServer.on("mche:candidate:send" /* CandidateSend */, async ({ receiver, candidate, sender }) => {
      if (receiver !== this.#id)
        return;
      if (this.#options.debug)
        log("Received candidate from", sender, "payload is", candidate);
      const connection = this.#connectionPool.get(sender);
      if (!connection)
        return;
      await connection.addIceCandidate(candidate);
    });
  }
  /**
   * Call this function to connection other peers.
   * @returns The ID of the peer. You need **hold** this ID to do next operations.
   */
  async #registerRemote(remoteId) {
    const nonNullRemoteId = remoteId || randomId();
    const connection = this.#getConnection();
    connection.onCandidate((candidate) => {
      const payload = {
        receiver: nonNullRemoteId,
        sender: this.#id,
        candidate
      };
      if (this.#options.debug)
        log("Sending candidate to", nonNullRemoteId, "payload is", payload);
      this.#signallingServer.send("mche:candidate:send" /* CandidateSend */, payload);
    });
    this.#connectionPool.set(nonNullRemoteId, connection);
    await this.#connect(nonNullRemoteId);
    return nonNullRemoteId;
  }
  async #connect(peerId) {
    const connection = this.#connectionPool.get(peerId);
    if (!connection) {
      error(new Error(`Peer ${peerId} not found.`));
      return;
    }
    const offer = await connection.createOffer();
    const payload = {
      offer,
      sender: this.#id,
      receiver: peerId
    };
    if (this.#options.debug)
      log("Sending offer to", peerId, "payload is", payload);
    this.#signallingServer.send("mche:offer:send" /* OfferSend */, payload);
  }
  // ----------------- Public API -----------------
  get id() {
    return this.#id;
  }
  broadcast(message) {
    const payload = {
      sender: this.#id,
      receiver: "broadcast",
      message
    };
    if (this.#options.debug)
      log("Broadcasting message", message, "payload is", payload);
    this.#connectionPool.forEach((connection) => {
      connection.addDataChannelCallback((channel) => {
        channel.send(JSON.stringify(payload.message));
      });
    });
  }
  /**
   * @returns cleanup fn
   */
  onBroadcast(callback) {
    const fn = (message) => {
      callback(message);
    };
    this.#dataBufferCallbacks.push(fn);
    return () => {
      const index = this.#dataBufferCallbacks.indexOf(fn);
      if (index !== -1)
        this.#dataBufferCallbacks.splice(index, 1);
    };
  }
  close() {
    this.#connectionPool.forEach((connection) => {
      connection.close();
    });
    this.#signallingServer.close();
  }
};
export {
  MCHelper
};
